# xper - Experiment Version Control

## What is Experiment Version Control?

Suppose you are about to come up with random ML (or any) experiments off the top of your head 
hoping that you would land on something that can be considered as novel and useful contribution 
to you PhD thesis. You do not know what ML architecture or what count of parameters or which 
hyperparameters will get you there, so you start with your initial experimental setup and run it. 
The results are not good enough obviously -- the experiment "fails" and now you have to run 
another one that is slightly or radically different than the one you just ran. Let's say you 
wonder what would happen if you changed a single line of code in the current experiment but you 
do not want to overwrite this experiment and its results either because you would like to get 
reminded how stupid you were (again) some time later when you decide to take a look at your 
previous experiments. This is when **xper** may come in handy! It's a tool that lets you 
to expand (for breadth-first exploration) and/or extend (for depth-first exploration) from 
the current experimental setup without overwriting anything.

```[zsh]
$ mkdir madness; cd madness # create your root directory for all the experiments
madness $ xper init # initialize the first experiment in subdirectory 001
madness $ xper tree # view experiment tree
  /001

madness/001 $ cd 001
madness/001 $ xper new # create a new experiment based on 001 -> initialized in subdirectory 002
mandness/001 $ xper tree
  /001
  /002

madness/002 $ cd ../002
madness/002 $ xper ext # explore slightly different version of 002 -> initialized in subdirectory 002/{001,002}
madness/002 $ xper tree
  /001
  /002
    /001
    /002

madness/002/002 $ cd 002 # madness/002/001/ has been initialized with the initial experiment previosly stored in madness/002/
madness/002/002 $ cd ../001 # go back to the initial experiment previously stored in madness/002/
madness/002/001 $ xper new # create a new experiment based on 002/001 -> initialized in subdirectory 002/003
madness/002/001 $ xper tree
  /001
  /002
    /001
    /002
    /003

madness/002/001 $ cd ../002 # go to the experiment 002/001
madness/002/002 $ xper ext # explore slightly different version of 002/001 -> initialized in subdirectory 002/001/{001,002}
madness/002/002 $ xper tree # view the experiment tree
  /001
  /002
    /001
    /002
      /001
      /002
    /003
```

## Experiment Version Control vs (Git) Version Control

Git's branching is cool but in order to do what the **xper** is doing, you would need to 
create branches (for different experiments) and constantly switch in between them. 
Git also introduces strong overhead for what the xper tries to do. With **xper**, you do not 
need to switch between different branches to rerun other versions of the experiments or 
just remind yourself how stupid you are. Everything is directory-based and you can switch 
between different experiments as easily as switching between different directories.

## TO DO

